"""Main executable module.

This module is the entry point of the coordinate canvas CLI utility. It
contains the CLI calls that initialize the utility.

Author:
    Paulo Sanchez (@erlete)
"""


from .interface.cli import cli

cli()
