"""Coordinate canvas package.

This package contains the coordinate canvas CLI utility, used for drawing
lines on a matplotlib plot for further coordinate extraction.

Author:
    Paulo Sanchez (@erlete)
"""
